@@ -2,7 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019      The Fluent Bit Authors
+ *  Copyright (C) 2019-2020 The Fluent Bit Authors
  *  Copyright (C) 2015-2018 Treasure Data Inc.
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
@@ -27,7 +27,7 @@
 #define FLB_SDS_H
 
 #include <fluent-bit/flb_info.h>
-
+#include <fluent-bit/flb_macros.h>
 #include <stdlib.h>
 #include <string.h>
 #include <inttypes.h>
@@ -51,6 +51,15 @@ static inline size_t flb_sds_len(flb_sds_t s)
     return (size_t) FLB_SDS_HEADER(s)->len;
 }
 
+static inline int flb_sds_is_empty(flb_sds_t s)
+{
+    if (flb_sds_len(s) == 0) {
+        return FLB_TRUE;
+    }
+
+    return FLB_FALSE;
+}
+
 static inline void flb_sds_len_set(flb_sds_t s, size_t len)
 {
     FLB_SDS_HEADER(s)->len = len;
@@ -78,10 +87,21 @@ static inline int flb_sds_cmp(flb_sds_t s, const char *str, int len)
     return strncmp(s, str, len);
 }
 
+static inline int flb_sds_casecmp(flb_sds_t s, const char *str, int len)
+{
+    if (flb_sds_len(s) != len) {
+        return -1;
+    }
+
+    return strncasecmp(s, str, len);
+}
+
 flb_sds_t flb_sds_create(const char *str);
 flb_sds_t flb_sds_create_len(const char *str, int len);
 flb_sds_t flb_sds_create_size(size_t size);
 flb_sds_t flb_sds_cat(flb_sds_t s, const char *str, int len);
+flb_sds_t flb_sds_cat_esc(flb_sds_t s, const char *str, int len,
+                                       char *esc, size_t esc_size);
 flb_sds_t flb_sds_cat_utf8(flb_sds_t *sds, const char *str, int len);
 flb_sds_t flb_sds_increase(flb_sds_t s, size_t len);
 flb_sds_t flb_sds_copy(flb_sds_t s, const char *str, int len);
