@@ -2,7 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019      The Fluent Bit Authors
+ *  Copyright (C) 2019-2020 The Fluent Bit Authors
  *  Copyright (C) 2015-2018 Treasure Data Inc.
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
@@ -85,6 +85,7 @@ struct flb_env *flb_env_create()
         return NULL;
     }
 
+    env->warn_unused = FLB_TRUE;
     env->ht = ht;
     env_preset(env);
 
@@ -143,7 +144,10 @@ const char *flb_env_get(struct flb_env *env, const char *key)
     /* If it was not found, try to get it from the real environment */
     out_buf = getenv(key);
     if (!out_buf) {
-        flb_warn("[env] variable ${%s} is used but not set", key);
+        return NULL;
+    }
+
+    if (strlen(out_buf) == 0) {
         return NULL;
     }
 
@@ -227,6 +231,9 @@ flb_sds_t flb_env_var_translate(struct flb_env *env, const char *value)
                 buf = s;
             }
         }
+        else if (env->warn_unused == FLB_TRUE) {
+            flb_warn("[env] variable ${%s} is used but not set", tmp);
+        }
         i += (v_start - (value + i)) + v_len;
     }
 
